"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("../src/index");
const assert_1 = require("assert");
const CLEAR = "\x1b[0m";
const BOLD = "\x1b[1m";
const FAIL = "\x1b[31m";
const OK = "\x1b[32m";
const RUN = "\x1b[33m";
class Test {
    constructor(tests) {
        this._tests = tests;
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const COUNT = '' + this._tests.length;
            // Find max name length for preattier printing
            let name_len = 0;
            for (let i = 0; i < this._tests.length; ++i) {
                let test = this._tests[i];
                name_len = Math.max(test.name.length, name_len);
            }
            for (let i = 0; i < this._tests.length; ++i) {
                let test = this._tests[i];
                process.stdout.write(`[${('' + (i + 1)).padStart(COUNT.length)}/${COUNT} ${RUN}${test.name.padEnd(name_len)}${CLEAR}] - ${BOLD}`);
                try {
                    let ret = test.run();
                    if (ret instanceof Promise) {
                        let wait = (promise) => __awaiter(this, void 0, void 0, function* () {
                            return promise;
                        });
                        yield wait(ret);
                    }
                    process.stdout.write(`${OK}OK`);
                }
                catch (error) {
                    process.stdout.write(`${FAIL}FAILED\n${error}`);
                }
                finally {
                    process.stdout.write(`${CLEAR}\n`);
                }
            }
        });
    }
}
let test = new Test([
    {
        name: 'null, undefined',
        run: () => {
            class Nullable extends index_1.JSONObject {
            }
            __decorate([
                index_1.optional,
                __metadata("design:type", String)
            ], Nullable.prototype, "value", void 0);
            let n_null = new Nullable({ value: null });
            assert_1.strict.ok(n_null.value === null);
            let n_undefiend = new Nullable({});
            assert_1.strict.ok(n_undefiend.value === undefined);
        }
    },
    {
        name: 'required',
        run: () => {
            class Book extends index_1.JSONObject {
            }
            __decorate([
                index_1.JSONObject.required,
                __metadata("design:type", String)
            ], Book.prototype, "name", void 0);
            const title = 'Moby Dick';
            let book = new Book({ name: title });
            assert_1.strict.ok(book.name == title);
            assert_1.strict.throws(() => {
                // Name is required
                let book = new Book;
            });
        }
    },
    {
        name: 'type',
        run: () => {
            class Book extends index_1.JSONObject {
            }
            __decorate([
                index_1.JSONObject.required,
                __metadata("design:type", String)
            ], Book.prototype, "name", void 0);
            const title = 'Moby Dick';
            let book = new Book({ name: title });
            assert_1.strict.ok(book.name == title);
            assert_1.strict.throws(() => {
                // Wrong type for name
                let book = new Book({ name: 12345 });
            });
        }
    },
    {
        name: 'optional',
        run: () => {
            class Book extends index_1.JSONObject {
            }
            __decorate([
                index_1.JSONObject.optional,
                __metadata("design:type", String)
            ], Book.prototype, "name", void 0);
            const title = 'Moby Dick';
            let book1 = new Book({ name: title });
            assert_1.strict.ok(book1.name == title);
            let book2 = new Book;
            assert_1.strict.ok(book2.name == undefined);
        }
    },
    {
        name: 'recursive',
        run: () => {
            class Author extends index_1.JSONObject {
            }
            __decorate([
                index_1.JSONObject.required,
                __metadata("design:type", String)
            ], Author.prototype, "name", void 0);
            class BookWithAuthor extends index_1.JSONObject {
            }
            __decorate([
                index_1.JSONObject.required,
                __metadata("design:type", String)
            ], BookWithAuthor.prototype, "name", void 0);
            __decorate([
                index_1.JSONObject.optional,
                __metadata("design:type", String)
            ], BookWithAuthor.prototype, "summary", void 0);
            __decorate([
                index_1.JSONObject.required,
                __metadata("design:type", Author)
            ], BookWithAuthor.prototype, "author", void 0);
            const title = 'Moby Dick';
            const author = "Herman Melville";
            let book = new BookWithAuthor({ name: title, author: { name: author } });
            assert_1.strict.ok(book.name == title);
            assert_1.strict.ok(book.author.name == author);
            assert_1.strict.ok(book.author instanceof Author);
        }
    },
    {
        name: 'unannonated',
        run: () => {
            class Person extends index_1.JSONObject {
            }
            let p = new Person({ id: 10 });
            assert_1.strict.ok(p.id === undefined);
        }
    },
    {
        name: 'lt-gt-ne',
        run: () => {
            class Person extends index_1.JSONObject {
            }
            __decorate([
                (0, index_1.lt)(10),
                (0, index_1.gt)(5),
                (0, index_1.ne)(6),
                __metadata("design:type", Number)
            ], Person.prototype, "id", void 0);
            assert_1.strict.throws(() => {
                let p = new Person({ id: 10 });
            });
            assert_1.strict.throws(() => {
                let p = new Person({ id: 5 });
            });
            assert_1.strict.throws(() => {
                let p = new Person({ id: 6 });
            });
            let p = new Person({ id: 7 });
            assert_1.strict.ok(p.id == 7);
        }
    },
    {
        name: 'lte',
        run: () => {
            class Person extends index_1.JSONObject {
            }
            __decorate([
                (0, index_1.lte)(5),
                __metadata("design:type", Number)
            ], Person.prototype, "id", void 0);
            assert_1.strict.throws(() => {
                let p = new Person({ id: 6 });
            });
            let p1 = new Person({ id: 5 });
            assert_1.strict.ok(p1.id == 5);
            let p2 = new Person({ id: 4 });
            assert_1.strict.ok(p2.id == 4);
        }
    },
    {
        name: 'gte',
        run: () => {
            class Person extends index_1.JSONObject {
            }
            __decorate([
                (0, index_1.gte)(5),
                __metadata("design:type", Number)
            ], Person.prototype, "id", void 0);
            assert_1.strict.throws(() => {
                let p = new Person({ id: 4 });
            });
            let p1 = new Person({ id: 5 });
            assert_1.strict.ok(p1.id == 5);
            let p2 = new Person({ id: 6 });
            assert_1.strict.ok(p2.id == 6);
        }
    },
    {
        name: 'eq',
        run: () => {
            class Person extends index_1.JSONObject {
            }
            __decorate([
                (0, index_1.eq)(10),
                __metadata("design:type", Number)
            ], Person.prototype, "id", void 0);
            assert_1.strict.throws(() => {
                let p = new Person({ id: 11 });
            });
            let p = new Person({ id: 10 });
            assert_1.strict.ok(p.id == 10);
        }
    },
    {
        name: 'minLength',
        run: () => {
            class Person extends index_1.JSONObject {
            }
            __decorate([
                (0, index_1.minLength)(5),
                __metadata("design:type", String)
            ], Person.prototype, "name", void 0);
            let p;
            assert_1.strict.throws(() => {
                p = new Person({ name: '1234' });
            });
            p = new Person({ name: '12345' });
            assert_1.strict.ok(p.name == '12345');
            p = new Person({});
            assert_1.strict.ok(p.name == undefined);
            p = new Person({ name: '1234567890' });
            assert_1.strict.ok(p.name == '1234567890');
            // Test length of arrays
            class ArrayTest extends index_1.JSONObject {
            }
            __decorate([
                (0, index_1.minLength)(2),
                (0, index_1.array)(String),
                __metadata("design:type", Array)
            ], ArrayTest.prototype, "test", void 0);
            let a;
            assert_1.strict.throws(() => {
                a = new ArrayTest({
                    test: ['1']
                });
            });
            a = new ArrayTest({
                test: ['a', 'b']
            });
            a = new ArrayTest({});
            assert_1.strict.ok(a.test == undefined);
            a = new ArrayTest({
                test: [
                    '1', '2', '3', '4'
                ]
            });
            assert_1.strict.ok(a.test[3] == '4');
        }
    },
    {
        name: 'maxLength',
        run: () => {
            class Person extends index_1.JSONObject {
            }
            __decorate([
                (0, index_1.maxLength)(5),
                __metadata("design:type", String)
            ], Person.prototype, "name", void 0);
            let p;
            assert_1.strict.throws(() => {
                p = new Person({ name: '123456' });
            });
            p = new Person({ name: '12345' });
            assert_1.strict.ok(p.name == '12345');
            p = new Person({});
            assert_1.strict.ok(p.name == undefined);
            p = new Person({ name: '1234' });
            assert_1.strict.ok(p.name == '1234');
            // test length of arrays
            class ArrayTest extends index_1.JSONObject {
            }
            __decorate([
                (0, index_1.maxLength)(2),
                (0, index_1.array)(String),
                __metadata("design:type", Array)
            ], ArrayTest.prototype, "test", void 0);
            let a;
            assert_1.strict.throws(() => {
                a = new ArrayTest({
                    test: ['1', '2', '3']
                });
            });
            a = new ArrayTest({
                test: ['a', 'b']
            });
            a = new ArrayTest({});
            assert_1.strict.ok(a.test == undefined);
            a = new ArrayTest({
                test: [
                    '1'
                ]
            });
            assert_1.strict.ok(a.test[0] == '1');
        }
    },
    {
        name: 'validate',
        run: () => {
            class Person extends index_1.JSONObject {
            }
            __decorate([
                (0, index_1.validate)((person, key, value) => {
                    if (value < 8)
                        throw new Error();
                }),
                __metadata("design:type", Number)
            ], Person.prototype, "id", void 0);
            let p = new Person({ id: 10 });
            assert_1.strict.ok(p.id == 10);
            assert_1.strict.throws(() => {
                let p = new Person({ id: 7 });
            });
        }
    },
    {
        name: 'custom',
        run: () => {
            class User extends index_1.JSONObject {
            }
            __decorate([
                index_1.required,
                __metadata("design:type", String)
            ], User.prototype, "specie", void 0);
            __decorate([
                (0, index_1.custom)((user, key, value) => {
                    // Translate to dog years if needed, using previously defiend values
                    return (user.specie == 'Canine') ? value * 7 : value;
                    // Note that user.name is not yet defined!
                }),
                index_1.required,
                __metadata("design:type", Number)
            ], User.prototype, "age", void 0);
            __decorate([
                index_1.required,
                __metadata("design:type", String)
            ], User.prototype, "name", void 0);
            let user1 = new User({ specie: 'Homo Sapiens', age: 28, name: 'Bob' });
            assert_1.strict.ok(user1.age == 28);
            let user2 = new User({ specie: 'Canine', age: 4, name: 'Fido' });
            assert_1.strict.ok(user2.age == 28);
        }
    },
    {
        name: 'custom assign',
        run: () => {
            class User extends index_1.JSONObject {
            }
            __decorate([
                index_1.required,
                __metadata("design:type", String)
            ], User.prototype, "specie", void 0);
            __decorate([
                (0, index_1.custom)((user, key, value) => {
                    // Translate to dog years if needed, using previously defiend values
                    user.realAge = (user.specie == 'Canine') ? value * 7 : value;
                    return value;
                }),
                index_1.required,
                __metadata("design:type", Number)
            ], User.prototype, "age", void 0);
            __decorate([
                index_1.required,
                __metadata("design:type", String)
            ], User.prototype, "name", void 0);
            let user1 = new User({ specie: 'Homo Sapiens', age: 28, name: 'Bob' });
            assert_1.strict.ok(user1.age == 28);
            assert_1.strict.ok(user1.realAge == 28);
            let user2 = new User({ specie: 'Canine', age: 4, name: 'Fido' });
            assert_1.strict.ok(user2.age == 4);
            assert_1.strict.ok(user2.realAge == 28);
        }
    },
    {
        name: 'union',
        run: () => {
            class Person extends index_1.JSONObject {
            }
            __decorate([
                index_1.required,
                (0, index_1.union)([1, 2, 3, 4, 5]),
                __metadata("design:type", Number)
            ], Person.prototype, "id", void 0);
            for (let i = 1; i <= 5; ++i) {
                let p = new Person({ id: i });
                assert_1.strict.ok(p.id == i);
            }
            assert_1.strict.throws(() => {
                let p = new Person({ id: 6 });
            });
        }
    },
    {
        name: 'map',
        run: () => {
            const title = 'Moby Dick';
            class Book extends index_1.JSONObject {
            }
            __decorate([
                (0, index_1.map)('title'),
                __metadata("design:type", String)
            ], Book.prototype, "name", void 0);
            let book = new Book({ title: title });
            assert_1.strict.ok(book.name == title);
            assert_1.strict.throws(() => {
                // Make 'name' required, make sure this fails
                class Book extends index_1.JSONObject {
                }
                __decorate([
                    (0, index_1.map)('title'),
                    index_1.required,
                    __metadata("design:type", String)
                ], Book.prototype, "name", void 0);
                let book = new Book({ name: title });
            });
        }
    },
    {
        name: 'passthrough',
        run: () => {
            class Car extends index_1.JSONObject {
            }
            __decorate([
                index_1.passthrough,
                __metadata("design:type", Object)
            ], Car.prototype, "userManual", void 0);
            let car = new Car({});
            assert_1.strict.ok(car.userManual === undefined);
            let car2 = new Car({ userManual: '12345' });
            assert_1.strict.ok(car2.userManual == '12345');
            let car3 = new Car({ userManual: {
                    pages: ['page1', 'page2']
                } });
            assert_1.strict.ok(car3.userManual.pages[0] == 'page1');
            assert_1.strict.ok(car3.userManual.pages[1] == 'page2');
        }
    },
    {
        name: 'default',
        run: () => {
            class Car extends index_1.JSONObject {
            }
            __decorate([
                (0, index_1.optional)(false),
                __metadata("design:type", Boolean)
            ], Car.prototype, "electric", void 0);
            let electric_car = new Car({ electric: true });
            assert_1.strict.ok(electric_car.electric);
            let carbon_emitting_car = new Car({});
            assert_1.strict.ok(carbon_emitting_car.electric === false);
        }
    },
    {
        name: 'integer',
        run: () => {
            class Int extends index_1.JSONObject {
            }
            __decorate([
                index_1.integer,
                __metadata("design:type", Number)
            ], Int.prototype, "value", void 0);
            let i = new Int({ value: 13 });
            assert_1.strict.throws(() => {
                let i = new Int({ value: 13.4 });
            });
        }
    },
    {
        name: 'array_simple',
        run: () => {
            class SimpleArrayTest extends index_1.JSONObject {
            }
            __decorate([
                (0, index_1.array)(String),
                __metadata("design:type", Array)
            ], SimpleArrayTest.prototype, "strings", void 0);
            __decorate([
                (0, index_1.array)(Number),
                __metadata("design:type", Array)
            ], SimpleArrayTest.prototype, "numbers", void 0);
            let sa1 = new SimpleArrayTest({ strings: ['a', 'b', 'c'] });
            assert_1.strict.throws(() => {
                let sa2 = new SimpleArrayTest({ strings: [1, 2, 3] });
            });
            let sa2 = new SimpleArrayTest({ numbers: [1, 2, 3] });
            assert_1.strict.throws(() => {
                let sa2 = new SimpleArrayTest({ numbers: ['a', 'b', 'c'] });
            });
        }
    },
    {
        name: 'array_object',
        run: () => {
            class Element extends index_1.JSONObject {
            }
            __decorate([
                index_1.optional,
                __metadata("design:type", Number)
            ], Element.prototype, "x", void 0);
            class ArrayTest extends index_1.JSONObject {
            }
            __decorate([
                (0, index_1.array)(Element),
                __metadata("design:type", Array)
            ], ArrayTest.prototype, "a", void 0);
            let json = { a: [{ x: 2 }, {}] };
            let arrayTest = new ArrayTest(json);
            assert_1.strict.ok(arrayTest.a[0] instanceof Element);
            assert_1.strict.ok(arrayTest.a[0].x == 2);
            assert_1.strict.ok(arrayTest.a[1].x === undefined);
            assert_1.strict.ok(arrayTest.a.length == 2);
        }
    },
    {
        name: 'array_passthrough',
        run: () => {
            // Simply type
            class ArrayTest extends index_1.JSONObject {
            }
            __decorate([
                index_1.required,
                __metadata("design:type", Array)
            ], ArrayTest.prototype, "a", void 0);
            let json = { a: [{ x: 2 }, {}] };
            let arrayTest = new ArrayTest(json);
            assert_1.strict.ok(arrayTest.a[0].x == 2);
            assert_1.strict.ok(arrayTest.a[1].x === undefined);
            assert_1.strict.ok(arrayTest.a.length == 2);
        }
    },
    {
        // Thanks @ste2425!
        // See https://github.com/moshegottlieb/ts-json-object/issues/4
        name: 'array_object_value',
        run: () => {
            // Simply type
            class ArrayTest extends index_1.JSONObject {
            }
            __decorate([
                index_1.optional,
                __metadata("design:type", Array)
            ], ArrayTest.prototype, "a", void 0);
            let json = { a: {} }; // object instead of array
            assert_1.strict.throws(() => {
                const arrayTest = new ArrayTest(json);
            });
        }
    },
    {
        // Thanks @ste2425!
        // See https://github.com/moshegottlieb/ts-json-object/issues/4
        // This test makes sure we didn't break anything while fixing #4
        name: 'array_object_value_null',
        run: () => {
            // Simply type
            class ArrayTest extends index_1.JSONObject {
            }
            __decorate([
                index_1.optional,
                __metadata("design:type", Array)
            ], ArrayTest.prototype, "a", void 0);
            let json = { a: null };
            const arrayTest1 = new ArrayTest(json);
            json = { a: undefined };
            const arrayTest2 = new ArrayTest(json);
        }
    },
    {
        // Thanks @ste2425!
        // See https://github.com/moshegottlieb/ts-json-object/issues/4
        // This test makes sure we didn't break anything while fixing #4
        name: 'array_object_value_required',
        run: () => {
            // Simply type
            class ArrayTest extends index_1.JSONObject {
            }
            __decorate([
                index_1.required,
                __metadata("design:type", Array)
            ], ArrayTest.prototype, "a", void 0);
            let json = {};
            assert_1.strict.throws(() => {
                const arrayTest = new ArrayTest(json);
            });
        }
    },
    {
        name: 'inheritance',
        run: () => {
            // Simply type
            class Base extends index_1.JSONObject {
            }
            __decorate([
                index_1.required,
                __metadata("design:type", Number)
            ], Base.prototype, "a", void 0);
            class Sub extends Base {
            }
            __decorate([
                index_1.required,
                __metadata("design:type", Number)
            ], Sub.prototype, "b", void 0);
            class Sub2 extends Base {
            }
            __decorate([
                index_1.required,
                __metadata("design:type", Number)
            ], Sub2.prototype, "a2", void 0);
            __decorate([
                (0, index_1.map)('x'),
                __metadata("design:type", String)
            ], Sub2.prototype, "b", void 0);
            let s = new Sub({ a: 1, b: 2 });
            assert_1.strict.ok(s.a == 1);
            assert_1.strict.ok(s.b == 2);
        }
    },
    {
        name: 'Date',
        run: () => {
            // Simply type
            class DateTest extends index_1.JSONObject {
            }
            __decorate([
                index_1.required,
                __metadata("design:type", Date)
            ], DateTest.prototype, "date", void 0);
            let test = new DateTest({ date: 1 });
            assert_1.strict.ok(Date.prototype.isPrototypeOf(test.date));
            assert_1.strict.throws(() => {
                let test = new DateTest({ date: 'not going to work' });
            });
        }
    },
    {
        name: 'Recursive',
        run: () => {
            class Node extends index_1.JSONObject {
                get relation() {
                    return this._relation;
                }
            }
            __decorate([
                index_1.required,
                __metadata("design:type", String)
            ], Node.prototype, "name", void 0);
            __decorate([
                (0, index_1.map)('relation'),
                (0, index_1.custom)((node, key, value) => {
                    return new Relation(value);
                }),
                __metadata("design:type", Object)
            ], Node.prototype, "_relation", void 0);
            class Relation extends index_1.JSONObject {
            }
            __decorate([
                index_1.required,
                __metadata("design:type", String)
            ], Relation.prototype, "type", void 0);
            __decorate([
                index_1.required,
                __metadata("design:type", Node)
            ], Relation.prototype, "node", void 0);
            const inner_name = 'TestName';
            let json = {
                type: 'test',
                node: {
                    name: 'name of node',
                    relation: {
                        type: 'test2',
                        node: {
                            name: inner_name
                        }
                    }
                }
            };
            let test = new Relation(json);
            assert_1.strict.ok(test.node.relation.node.name == inner_name);
        }
    }
]);
test.run();
//# sourceMappingURL=index.js.map